;;;; cxx.el -- C++ related lisp functions

;;; Code

;; required modules
(require 'cc-mode)


;; Customization Variables
(defgroup cxx nil
  "*C++ related functions."
  :group 'emacs)


(defcustom cxx-user-id nil
  "*My user id.
If empty, will use value of environment variable USERNAME."
  :type 'string
  :group 'cxx-stuff)

(defun cxx-user-id ()
  "Return value of cxx-user-id variable if non null,
else return value of environment variable USERNAME."
  (or (and cxx-user-id (> 0 (length cxx-user-id)))
      (setq cxx-user-id (getenv "USERNAME")))
  cxx-user-id)


(defcustom cxx-time-format "%Y-%m-%d"
  "*Time format string available for use by cxx module.

For more information on the format, see the documentation of the
function format-time-string."
  :type 'string
  :group 'cxx-stuff)

(defcustom cxx-auto t
  "*Automatically call cxx-header or cxx-source when user opens new file."
  :type 'boolean
  :group 'cxx)

(defcustom cxx-class-template-file (expand-file-name "~/lisp/class-tmpl")
  "*C++ class template file."
  :type 'file
  :group 'cxx)

(defcustom cxx-header-template-file (expand-file-name "~/lisp/header-tmpl")
  "*C/C++ header template file."
  :type 'file
  :group 'cxx)

(defcustom cxx-source-template-file (expand-file-name "~/lisp/source-tmpl")
  "*C/C++ source template file"
  :type 'file
  :group 'cxx)

(defcustom cxx-rh-begin "/*** Revision History:"
  "*String indicating beginning of revision history section."
  :type 'string
  :group 'revision-history)

(defcustom cxx-rh-end "***/"
  "*String indicating end of revision history section."
  :type 'string
  :group 'revision-history)

(defcustom cxx-rh-template-file (expand-file-name "~/lisp/rh-tmpl")
  "*Revision history entry template file."
  :type 'file
  :group 'revision-history)



;; add hooks
(add-hook 'find-file-not-found-hooks 'cxx-header-wrapper)
(add-hook 'find-file-not-found-hooks 'cxx-source-wrapper)


;; menus
(defvar cxx-menus-added nil)

(defun cxx-add-menus ()
  (or cxx-menus-added
      (progn
	(add-menu-button '("C++") "------")
	(add-menu-button '("C++") [ "Header" cxx-header ])
	(add-menu-button '("C++") [ "Source" cxx-source ])
	(add-menu-button '("C++") [ "Class" cxx-class ])
	(add-menu-button '("C++") [ "Revision History Add" cxx-rh-add ])
	(setq cxx-menus-added t))))

(add-hook 'c-mode-common-hook 'cxx-add-menus)



;; keybindings
(add-hook 'c-mode-common-hook 'cxx-keybindings)

(defun cxx-keybindings ()
  (local-set-key "\C-ca" 'cxx-rh-add)
  (local-set-key "\C-cc" 'cxx-class)
  (local-set-key "\C-ce" 'cxx-doxy-end-comment))


;;; functions
(defun cxx-class (name desc)
  "create C++ class declaration from template"
  (interactive "sName: \nsDescription: ")
  (fill-template cxx-class-template-file
                  "<name>" name
                  "<desc>" desc)
  ;; kludge to get indenting to work right:
  (save-excursion
    (search-forward "public:")
    (c-indent-defun)))


(defun cxx-header ()
  "create C/C++ header file from template"
  (interactive)
  (let ((name (read-string "Header file name: " (file-name-nondirectory (buffer-file-name))))
         (desc (read-string "Description: ")))
    (fill-template cxx-header-template-file
                    "<name>" name
                    "<desc>" desc
                    "<preproc>" (concat "__" (upcase (replace-in-string name ?. ?_)) "__")
                    "<author>" (cxx-user-id)
                    "<date>" (current-time-string)
                    "<year>" (cxx-year))))

                                             
(defun cxx-header-wrapper ()
  "prompt user to see if he wants a header file created"
  (if (and cxx-auto (or (ext-match "h") (ext-match "hpp")) (y-or-n-p "Create header file? "))
      (cxx-header)))


(defun cxx-source ()
  "create C/C++ source file from template"
  (interactive)
  (let ((name (read-string "Source file name: " (file-name-nondirectory (buffer-file-name))))
         (desc (read-string "Description: ")))
  (fill-template cxx-source-template-file
                  "<name>" name
                  "<desc>" desc
                  "<author>" (cxx-user-id)
                  "<date>" (current-time-string)
                  "<year>" (cxx-year))))
                  

(defun cxx-source-wrapper ()
  "prompt user to see if he wants a source file created"
  (if (and cxx-auto (or (ext-match "c") (ext-match "cc") (ext-match "C") (ext-match "cpp"))
            (y-or-n-p "Create source file? "))
      (cxx-source)))


(defun cxx-rh-add (comment)
  "Add item to revision history."
  (interactive "sComment: ")
  (save-excursion
    (goto-char (point-min))
    (if (not (re-search-forward (escapify-regx-char cxx-rh-begin) nil t))
        (progn
          (save-excursion
            (re-search-forward (escapify-regx-char cxx-rh-end) nil t)
            (insert "\n\n")
            (cxx-rh-section))
          (re-search-forward (escapify-regx-char cxx-rh-begin))))
    (re-search-forward (escapify-regx-char cxx-rh-end))
    (beginning-of-line)
    (fill-template cxx-rh-template-file
                   "<date>" (format-time-string cxx-time-format)
                   "<user>" (cxx-user-id)
                   "<comment>" comment)
    (re-search-forward (escapify-regx-char cxx-rh-end))
    (goto-char (match-beginning 0))
    (insert "\n\n")))


(defun cxx-rh-section ()
  "Add revision history section at current point in file."
  (interactive)
  (save-excursion
    (insert (concat cxx-rh-begin "\n\n" cxx-rh-end "\n\n")))
  (font-lock-fontify-buffer))


(defun cxx-year ()
  "Return current year."
  (format-time-string "%Y"))


(defun cxx-doxy-end-comment ()
  "Add Doxygen short end of line comment."
  (interactive)
  (indent-for-comment)
  (search-backward "//")
  (forward-char 2)
  (insert "!< "))


;; provide 
(provide 'cxx)

;;; cxx.el ends here
